function [Et, t, Ew, w ] = pulsegenerator(N0,shapefunction,FWHM,deltat,lambda,temporalchirp,SPM,spectralchirp)
%   PULSEGENERATOR Generates a pulse of an inputted function with the given parameters.
%
%       [Et, t, Ew, w ] = PULSEGENERATOR([])
%
%   PULSEGENERATOR(N) returns an N point random function.
%   PULSEGENERATOR([N, m]) returns an N point random function, with the random generator
%   initialized in state m.
%
%   PULSEGENERATOR(N, @shapefunction, FWHM, deltat, lambda) returns an N point
%       function  of type shapefunction or the rand() function.  
%
%           Shapefunction Definition
%               [Eenv, t] = shapefunction(FWHM,N,deltat)
%           
%
%
%       FWHM is the length of the function in fs; deltat is the 
%       time increment in fs; lambda is the center wavelength 
%       in nm. 
%
%   PULSEGENERATOR([],@shapefunction,[],[],[],temporalchirp,SPM) returns a pulse
%       of type shapefunction with a polynomial temporal chirp.  Temporal chirp
%       is of form [A,B,C,...] which corresponds to the polynomial:
%
%                       A*x^0 + B*x^1 + C*x^2 + ...
%
%       SPM is Self-Phase Modulation. temporal phase = SPM.*abs(Eenv).^2
%
%   PULSEGENERATOR([],@shapefunction,[],[],[],[],[],spectralchirp) returns a
%       pulse of type shapefunction with a polynomil spectral chirp. Spectral chirp
%       is of form [A,B,C,...] which corresponds to the polynomail:
%
%                       A*x^0 + B*x^1 + C*x^2 + ...
%
%   Defaults: N = 1024; shapefunction = rand(); FWHM = 40; deltat = 1; lamba = 800

%	$Revision: 1.11 $ $Date: 2002/06/24 20:36:59 $
%
%	$Log: pulsegenerator.m,v $
%	Revision 1.11  2002/06/24 20:36:59  xg
%	Random generator initialization enabled.
%	
%	Revision 1.10  2002/04/30 22:40:45  zeekec
%	*** empty log message ***
%	
%	Revision 1.9  2002/04/02 21:26:19  zeekec
%	The "rand" option now outputs a number with random amplitude (0,1) and phase (-pi,pi).
%	
%	Revision 1.8  2001/10/31 18:07:30  zeekec
%	Fixed error in SPM.
%	
%	Revision 1.7  2001/10/23 16:39:17  zeekec
%	Fixed error in spectral phase.
%	
%	Revision 1.6  2001/10/18 19:32:11  zeekec
%	More cleanups.
%	
%	Revision 1.5  2001/10/18 19:21:39  zeekec
%	Readded fliplr.
%	
%	Revision 1.4  2001/10/16 18:26:53  zeekec
%	Removed some temporaries, fixed a miss spelling, general imporvements.
%	
%	Revision 1.3  2001/09/05 01:21:46  zeekec
%	Added checks for empty inputs.
%	
%	Revision 1.2  2001/07/10 01:10:00  zeekec
%	Library cleanup.  Added, deleted, and moved files.
%	
%	v0.01, 7/2/01, Michael Butterfield, <gte881s@prism.gatech.edu>
%	v0.01, 6/28/01, Michael Butterfield, <gte881s@prism.gatech.edu>
%

error(nargchk(0,8,nargin)); 

if nargin < 1 | isempty(N0)
	N0 = 1024;    
end
if nargin < 3  | isempty(FWHM) | FWHM == 0
	FWHM = 40;
end
if nargin < 4  | isempty(deltat) | deltat == 0
	deltat = 1;
end
if nargin < 5  | isempty(lambda)
	lambda = 800; % in nm.
end

if (length(N0) == 1)
    N = N0;
elseif (length(N0) == 2)
    N = N0(1);
    rand('state', N0(2));
else
    error('First parameter too long.');
end

% calculate the envelope of the pulse
t = (-N/2 : N/2 - 1) * deltat;

if (nargin < 2)  | isempty(shapefunction) | strcmp(func2str(shapefunction),'rand')
	Et = rand(size(t)) .* exp(i*2*pi*rand(size(t)));
else
	Et = feval(shapefunction,FWHM,N,deltat);
end

% calculate the temporal phase
if nargin > 5 & ~isempty(temporalchirp) & any(temporalchirp)
	tphase = polyval(fliplr(temporalchirp),t);
	
	% apply the phase to the pulse
	Et= Et.*exp(i.*tphase);
end

if nargin > 6 & ~isempty(SPM) & SPM ~= 0;
	tphase = SPM .* abs(Et) .^ 2;
	Et= Et.*exp(i.*tphase);
end

% center omega = 2 pi c/lambda

omega0 = ltow(lambda);

% calculate Ew from Et
Ew = fftc(Et);

% calculate w
deltaomega = 2*pi/(t(end) - t(1));

w = (-N/2:N/2-1)*deltaomega + omega0;

% add spectral phase
if nargin > 7 & ~isempty(spectralchirp) & any(spectralchirp)
	sphase = polyval(fliplr(spectralchirp),w-omega0);
	Ew = Ew .* exp(i * sphase);
	Et = ifftc(Ew);
end

